#version 3.61;

#include "shapes.inc"
#include "metals.inc"
#include "glass.inc"
#include "colors.inc"

#declare Use_Photons = 3;
#declare Load_Photons = 0;
#declare Photon_File = "laser_test.ph";


/*
photons {
  spacing <photon_spacing> | count <photons_to_shoot>

  [gather <min_gather>, <max_gather>]
  [media <max_steps> [,<factor>]]
  [jitter <jitter_amount>]
  [max_trace_level <photon_trace_level>]
  [adc_bailout <photon_adc_bailout>]
  [save_file "filename" | load_file "filename"]
  [autostop <autostop_fraction>]
  [expand_thresholds <percent_increase>, <expand_min>]
  [radius <gather_radius>,<multiplier>,
          <gather_radius_media>,<multiplier>]
}
*/

global_settings {
  assumed_gamma 1.0
  max_trace_level 7
  #if(Use_Photons = 1)
    photons {
      spacing 0.05
      media 200
      max_trace_level 10
      adc_bailout 1/256
      #if(Load_Photons & file_exists(Photon_File))
        load_file Photon_File
      //#else
      //  save_file Photon_File
      #end
    }
  #else
  #if(Use_Photons = 2)
    photons {
      spacing 0.075
      media 150
      max_trace_level 7
      adc_bailout 1/200
      #if(Load_Photons & file_exists(Photon_File))
        load_file Photon_File
      //#else
      //  save_file Photon_File
      #end
    }  
  
  #else
  #if(Use_Photons = 3)
    photons {
      spacing 0.1
      media 80
      max_trace_level 5
      adc_bailout 1/64
      #if(Load_Photons & file_exists(Photon_File))
        load_file Photon_File
      //#else
      //  save_file Photon_File
      #end
    }
  #end
  #end
  #end
}

camera {
  location <0, 70, -150>
  up y
  right x * image_width/image_height
  look_at 0

  angle 15
}

#declare laser =
light_source {
  <15,2,22>, <.4,0,1>*90
  cylinder
  point_at <-20,2,22>
  radius .1
  falloff .4
  tightness 0.25
  photons {
    reflection on
    refraction on
  }
}
#declare laser2 =
light_source {
  <-50,2,-10>, <.3,0,1>*40
  cylinder
  point_at <20,2,-10>
  radius 1
  falloff 2
  tightness 0.25
  photons {
    reflection on
    refraction on
  }
}


#declare reflector = object {
Round_Box_Union(<-0.8, 0, -4>, <0, 4, 4>, 0.25)

  pigment { rgb .2 }
  finish {
    ambient .1
    diffuse .6
    reflection { 1 }
  } 
  photons {
    target
    reflection on
    refraction on
  }
  
}

#declare _z_ = -<44,36,0>;
#declare _x_ = vnormalize(vcross(y,_z_))*10;
#declare _y_ = vnormalize(vcross(_z_,_x_))*10;

//light_group {
#if (10)
  light_source { <44,36,0> color rgb <.8,.8,.99>*1.2
  fade_distance vlength(<44,36,0>)-4 fade_power 5
  
    area_light _x_,_y_, 13,13 adaptive 0 jitter orient circular
    media_interaction off media_attenuation off
  }
#else #if (0)
  light_source {
   <27.5,36.1,-43.8> rgb <.82,.88,.99>*.6
   media_interaction off media_attenuation off 
  }  
#end  #end
//}

//#declare lightgun =
merge {
  cylinder { <0,0,0>, <0,4,0> 1  open rotate z*-90 }
  cylinder { <0,-1,0>, <0,1.5,0> 0.7 translate <2,-2,0> }

  texture { T_Chrome_3C } 
  translate <15.2,2,22>
}

#declare tiletex =
texture {
    pigment { uv_mapping
      gradient x
      pigment_map{
        [1/20 rgb 0][1/20 gradient y
          colour_map{
            [1/20 rgb 0]
            [1/20 rgb <0.98431, 0.98431, 0.98431>]
          }
        ]
      }
      scale -<1/80,1/180,1>
    }
    finish { ambient 0 diffuse 0.6}
}
//#declare table  =
object {
  Round_Box_Union(<-90, -1.6, -90>, <90, -1, 900>, 0.25)
 // pigment { Gray60 }
 // finish {  ambient .1 diffuse .6 }
 texture { tiletex }
}


//right side at zero
//object { reflector rotate y*45 translate z*22}
//object { reflector rotate y*-135 }


#declare trget = box {
  <-0.5, -8, -8>, <0.5, 8, 8>
  pigment { White }
  rotate z * 90
  translate  <-9, 9, 0>
}

#declare Scatter_M =
  material {
    texture {
      pigment { rgbt 1 }
      finish {
        ambient 0
        diffuse 0
      }
    }
    interior {
      media {
        method 3
        intervals 1
        samples 70, 1000
        scattering {
          1, rgb 0.1
          extinction 0.00001
        }
      }
    }
  }
#declare m_glass = 
material { texture {
    pigment { rgbf <1.0, 1.0, 1.0,.9> }
    finish { F_Glass3 } }
  interior { I_Glass }
}
  
#declare lens_magnification = function(f,s1) {
 f / (f-s1)
}
//http://en.wikipedia.org/wiki/Lens_(optics)
#declare lens_focal_len = function(r1,r2,_ior,d) {
  // Thin lens equation
  // 1 / ((_ior-1) * ((1 / r1)-(1 / r2)))
  1 / ((_ior-1) * (((1 / r1)-(1 / r2)) + (((_ior-1) * d) / (_ior * r1 * r2))  ))
}
/*alias function-curve {
  ; Fx: X + (Y*R/B) = R
  ; r=xwidthradius b=yheightofarc and r==b for circle
} */
#declare quadx = function(wrad,hrad,yinput) {
  // (r,b,y)
  sqrt((wrad*wrad) - pow(yinput*wrad/hrad,2))
  // x = sqrt(r - (y*r/b))
}
#declare quady = function(wrad,hrad,xinput) {
sqrt(wrad*wrad-xinput*xinput) * (hrad / wrad)
  // y = sqrt(r - x) * (b/r)
} 
#declare maglass = 
intersection {
sphere {  <3.2,0,0>, 4 }
sphere {  <-3.2,0,0>, 4 }
  material { m_glass } photons { target reflection on refraction on }
}
//object { maglass translate <-6,2,-10> }




#declare concav_glass_ = 
difference { cylinder { <-.5,0,0>,<.5,0,0> 3 }
  sphere { <-4.0005,0,0>, 4 } sphere { <4.0005,0,0>, 4 }
  material { m_glass } photons { target reflection on refraction on }
}
object { concav_glass_ translate <-8+4.137943+0.291901,2,-10> }
// about
//(4-quadx(4,4,1.5)) + (lens_focal_len(8,-8,1.5,1.6) / 2)

#declare maglass_ = 
intersection {
sphere {  <7.2,0,0>, 8 }
sphere {  <-7.2,0,0>, 8 }
  material { m_glass } photons { target reflection on refraction on }
}
object { maglass_ translate <-8,2,-10> }

#declare con_reflector = 
difference {
  sphere {  0, 4 }
  box { <-5, -5, -2>, <5, 5, 5> }

  pigment { rgb .2 }
  finish {
    ambient .1
    diffuse .6
    reflection { 1 }
  } 
  photons {
    target
    reflection on
    refraction on
  }

}
#declare concav_reflector = 
difference {
  sphere {  0, 4 }
  sphere {  0, 5 translate z*2 }

  pigment { rgb .2 }
  finish {
    ambient .1
    diffuse .6
    reflection { 1 }
  } 
  photons {
    target
    reflection on
    refraction on
  }

}

object { con_reflector 
  translate z*2
  rotate y*135
  translate <0,2,22>
}

object { con_reflector 
  translate z*2
  rotate y*-45
  translate <0,2,0>
}

object { con_reflector 
  translate z*2
  rotate y*135
  translate x*-15
  translate <0,2,0>
}
#declare concav_ =
object { concav_reflector
  translate z*3
  rotate y*45
  translate x*-15 translate z*-10
  translate <0,2,0>
}


#declare container =
sphere {  0, 60
  hollow
  material { Scatter_M }
  photons {
    pass_through
  }
}
object { container }


#declare _prism = prism {
linear_sweep
bezier_spline
0.44, //top
0.0, //bottom
12 ,
<0.22,0>,<-0.22,0>,<0.22,0>,<-0.22,0>,
<-0.22,0>,<0.0,0.38106>,<-0.22,0>,<0.0,0.38106>,
<0.0,0.38106>,<0.22,0>,<0.0,0.38106>,<0.22,0>
}

#declare aprism =
object { _prism scale 10 
  texture {
    pigment { rgbf <1.0, 1.0, 1.0,.9> }
    finish { F_Glass3 }
  }
  interior { I_Glass }

  photons {
    target
    reflection on
    refraction on
  }
}
//object { aprism rotate y*135 translate <0,0,22> }
//object { aprism translate <6,0,-12> }



//light_source { laser }
light_source { laser2 }
